package com.ningdatech.pmapi.projectdeclared.manage;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.pmapi.common.enumeration.ProjectProcessStageEnum;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.datascope.model.DataScopeDTO;
import com.ningdatech.pmapi.datascope.utils.DataScopeUtil;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.projectdeclared.model.dto.ProjectConditionDTO;
import com.ningdatech.pmapi.projectdeclared.model.po.DeclaredProjectStatisticsPO;
import com.ningdatech.pmapi.projectdeclared.service.IDeclaredStatisticsService;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.bean.entity.WflowModels;
import com.wflow.bean.entity.WflowOrgModels;
import com.wflow.enums.OrgTypeEnum;
import com.wflow.enums.ProcessDefTypeEnum;
import com.wflow.exception.BusinessException;
import com.wflow.service.OrgProcdefService;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.dto.ProcessInstanceUserDto;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname DefaultDeclaredProjectManage
 *
 * @Description
 * @Date 2023/2/14 17:12
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class DefaultDeclaredProjectManage {

    private final IProjectService projectService;

    private final UserInfoHelper userInfoHelper;

    private final IDingOrganizationService dingOrganizationService;
    private final OrgProcdefService orgProcdefService;

    private final ProcessModelService processModelService;

    private final ProcessInstanceService processService;

    private final IProjectInstService projectInstService;

    private final IDeclaredStatisticsService statisticsService;

    //项目名称去重
    public void checkDuplication(ProjectDTO project){
        VUtils.isTrue(projectService.count(Wrappers.lambdaQuery(Project.class)
                        .eq(Project::getProjectName,project.getProjectName())
                        .ne(Objects.nonNull(project.getId()),Project::getId,project.getId())) > 0)
                .throwMessage(String.format("修改失败 此项目名 【%s】 已存在!",project.getProjectName()));
    }

    public void checkAmount(ProjectDTO projectInfo) {
        if(Objects.nonNull(projectInfo.getDeclareAmount())){
            BigDecimal declareAmount = projectInfo.getDeclareAmount();
            BigDecimal govSuperFinanceAmount = projectInfo.getDeclareGovSuperiorFinanceAmount();
            BigDecimal govOwnFinanceAmount = projectInfo.getDeclareGovOwnFinanceAmount();
            BigDecimal bankLendingAmount = projectInfo.getDeclareBankLendingAmount();
            BigDecimal haveAmount = projectInfo.getDeclareHaveAmount();
            BigDecimal otherAmount = projectInfo.getDeclareOtherAmount();
            BigDecimal totalAmount = govSuperFinanceAmount
                    .add(govOwnFinanceAmount).add(bankLendingAmount).add(haveAmount).add(otherAmount);
            VUtils.isTrue(declareAmount.compareTo(totalAmount) != 0)
                    .throwMessage(String.format("申报失败! 申报总金额【%s】 不等于其它申报金额 【%s】",declareAmount,totalAmount));

            //判断 年度计划金额 如果开着的话
            if(projectInfo.getIsAnnualPlanAmount()){
                BigDecimal annualPlanAmount = projectInfo.getAnnualPlanAmount();

                VUtils.isTrue(annualPlanAmount.compareTo(BigDecimal.ZERO) <= 0)
                        .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不能小于等于0",annualPlanAmount));
                VUtils.isTrue(annualPlanAmount.compareTo(declareAmount) > 0)
                        .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不能大于申报金额 【%s】",annualPlanAmount,declareAmount));

                BigDecimal annualGovSuperAmount = projectInfo.getAnnualPlanGovSuperiorFinanceAmount();
                BigDecimal annualOwnSuperAmount = projectInfo.getAnnualPlanGovOwnFinanceAmount();
                BigDecimal annualBankLendingAmount = projectInfo.getAnnualPlanBankLendingAmount();
                BigDecimal annualHaveAmount = projectInfo.getAnnualPlanHaveAmount();
                BigDecimal anualOtherAmount = projectInfo.getAnnualPlanOtherAmount();
                BigDecimal totalAnnual = annualGovSuperAmount
                        .add(annualOwnSuperAmount).add(annualBankLendingAmount).add(annualHaveAmount).add(anualOtherAmount);
                VUtils.isTrue(annualPlanAmount.compareTo(totalAnnual) != 0)
                        .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不等于其它年度支付金额 【%s】",annualPlanAmount,totalAnnual));
            }
        }
    }

    public ProcessInstanceUserDto buildUser(String employeeCode){
        UserFullInfoDTO userInfoDetail = userInfoHelper.getUserFullInfoByEmployeeCode(employeeCode);

        VUtils.isTrue(Objects.isNull(userInfoDetail))
                .throwMessage(String.format("【%s】获取不到用户!",employeeCode));

        return ProcessInstanceUserDto.builder()
                .userId(String.valueOf(userInfoDetail.getEmployeeCode()))
                .userName(userInfoDetail.getRealName())
                .orgCode(userInfoDetail.getOrganizationCode())
                .orgName(userInfoDetail.getOrganizationName())
                .build();
    }

    public Map<String, OrgInfoDTO> buildOrgModelMap(String employeeCode,Project project) {
        Map<String, OrgInfoDTO> orgMap = new HashMap<>();

        // 查出所有的单位流程配置
        List<WflowOrgModels> orgModelsList = orgProcdefService.list();

        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfoByEmployeeCode(employeeCode);
        if(Objects.isNull(userFullInfo)){
            log.warn("取不到员工 返回空map");
            return Collections.emptyMap();
        }
        DingOrganization startOrg = dingOrganizationService.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, userFullInfo.getOrganizationCode()));
        String startOrgCode = startOrg.getOrganizationCode();
        String startOrgName = startOrg.getOrganizationName();
        String startOrgParentCode;
        String startOrgParentName;
        // 如果申报时 上级主管单位,由该单位自己审核
        if (Objects.isNull(project.getSuperOrgCode())){
            startOrgParentCode = startOrgCode;
            startOrgParentName = startOrgName;
        }else {
            startOrgParentCode = project.getSuperOrgCode();
            startOrgParentName = project.getSuperOrg();
        }
        //查询 当前发起人及主管单位所在区域的 单位流程配置
        OrgInfoDTO startOrgInfoDto = new OrgInfoDTO();
        startOrgInfoDto.setOrganizationCode(startOrgCode);
        startOrgInfoDto.setOrganizationName(startOrgName);

        startOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        OrgInfoDTO parentOrgInfoDto = new OrgInfoDTO();
        parentOrgInfoDto.setOrganizationCode(startOrgParentCode);
        parentOrgInfoDto.setOrganizationName(startOrgParentName);
        parentOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgParentCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        orgMap.put(OrgTypeEnum.TARGET_OWNER.name(),startOrgInfoDto);
        orgMap.put(OrgTypeEnum.TARGET_MANAGEMENT.name(),parentOrgInfoDto);

        // 如果是指定单位，直接根据流程定义ID放入map
        OrgInfoDTO orgInfoDTO = new OrgInfoDTO();
        orgInfoDTO.setOrgModelMap(orgModelsList.stream()
                .collect(Collectors.toMap(WflowOrgModels::getProcessDefId, v -> v)));
        orgMap.put(OrgTypeEnum.TARGET_LABEL.name(),orgInfoDTO);
        return orgMap;
    }

    //直接提交预审方法 提取 在省级联审通过的时候 也可以用
    public String directStartProcess(Project projectInfo,String employeeCode){
        VUtils.isTrue(Objects.isNull(employeeCode))
                .throwMessage("发起人 员工code 不能为空!");

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(buildUser(employeeCode));
        params.setProcessUsers(Collections.emptyMap());
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(projectInfo, conditionDto);
        params.setFormData(JSON.parseObject(JSON.toJSONString(conditionDto), new TypeReference<Map<String, Object>>() {
        }));

        String regionCode = projectInfo.getAreaCode();

        WflowModels model = processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getRegionCode, regionCode)
                .eq(WflowModels::getProcessType, ProjectProcessStageEnum.PROJECT_PREQUALIFICATION_APPROVAL_PROCESS.getCode())
                .last("limit 1"));

        if (Objects.isNull(model)) {
            log.error("此 【{}】区域找不到 预审流程配置", regionCode);
            throw new BusinessException(String.format("此 【%s】区域找不到 预审流程配置", regionCode));
        }

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Map<String, OrgInfoDTO> orgModelMap = buildOrgModelMap(employeeCode,projectInfo);
        String instanceId = processService.newStartProcess(model.getProcessDefId(),model.getFormId(), params,orgModelMap);
        log.info("提交预审项目成功 【{}】", instanceId);

        //保存预审项目
        preModifyProject(projectInfo, instanceId);

        return instanceId;
    }

    /**
     * 提交预审项目 时 更新信息
     *
     * @param project
     * @param instanceId
     */
    private void preModifyProject(Project project, String instanceId) {
        //流程启动之后 入库项目 重要业务信息 用于列表查询 展示
        try {
            project.setUpdateOn(LocalDateTime.now());
            project.setInstCode(instanceId);
            projectService.updateById(project);

            //保存项目和实例的关系
            ProjectInst projectInst = new ProjectInst();
            projectInst.setProjectId(project.getId());
            projectInst.setInstCode(instanceId);
            projectInst.setCreatOn(LocalDateTime.now());
            projectInst.setUpdateOn(LocalDateTime.now());
            projectInst.setInstType(ProjectProcessStageEnum.PROJECT_PREQUALIFICATION_APPROVAL_PROCESS.getCode());
            projectInstService.save(projectInst);
        } catch (Exception e) {
            log.error("提交预审 项目信息修改 错误 ", e);
            throw new BusinessException("提交预审 项目信息修改 错误 :" + e.getMessage());
        }
    }

    //根据提交者的单位
    public DeclaredProjectStatisticsPO declaredProjectOrgStatistics(Integer year){
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        //查此人建设单位的项目
        return statisticsService.getOrgStatistics(user.getOrganizationCode(),year);
    }

    //根据提交者的区域 他是区管或者超管
    public DeclaredProjectStatisticsPO declaredProjectRegionStatistics(Integer year){
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        //查此人建设单位的项目
        return statisticsService.getRegionStatistics(user.getRegionCode(),year);
    }
}
